<?php

namespace Drupal\Tests\box\Traits;

use Drupal\box\Entity\Box;

/**
 * Provides methods to create box based on default settings.
 *
 * This trait is meant to be used only by test classes.
 */
trait BoxCreationTrait {

  /**
   * Get a box from the database based on its title.
   *
   * @param string|\Drupal\Component\Render\MarkupInterface $title
   *   A box title, usually generated by $this->randomMachineName().
   * @param $reset
   *   (optional) Whether to reset the entity cache.
   *
   * @return \Drupal\box\Entity\BoxInterface
   *   A box entity matching $title.
   */
  public function getBoxByTitle($title, $reset = FALSE) {
    if ($reset) {
      \Drupal::entityTypeManager()->getStorage('box')->resetCache();
    }
    // Cast MarkupInterface objects to string.
    $title = (string) $title;
    $boxes = \Drupal::entityTypeManager()
      ->getStorage('box')
      ->loadByProperties(['title' => $title]);
    // Load the first box returned from the database.
    $returned_box = reset($boxes);
    return $returned_box;
  }

  /**
   * Creates a box based on default settings.
   *
   * @param array $settings
   *   (optional) An associative array of settings for the box, as used in
   *   entity_create(). Override the defaults by specifying the key and value
   *   in the array, for example:
   *   @code
   *     $this->drupalCreateBox(array(
   *       'title' => t('Hello, world!'),
   *       'type' => 'cardboard',
   *     ));
   *   @endcode
   *   The following defaults are provided:
   *   - title: Random string.
   *   - type: 'default'.
   *   - uid: The currently logged in user, or anonymous.
   *
   * @return \Drupal\box\Entity\Box
   *   The created box entity.
   */
  protected function createBox(array $settings = []) {
    // Populate defaults array.
    $settings += [
      'machine_name' => [
        [
          'value' => $this->randomMachineName(32),
        ],
      ],
      'title'     => $this->randomMachineName(8),
      'type'      => 'default',
      'uid'       => \Drupal::currentUser()->id(),
    ];
    $box = Box::create($settings);
    $box->save();

    return $box;
  }

}
